'use strict';
/**
 * NODE端请求
 */

Object.defineProperty(exports, '__esModule', {
  value: true
});

const server_1 = require('award-utils/server');

const APIGateway_1 = require('../../utils/APIGateway');

const apiGatewayConfig = require('../../config/apiGateway');

const log_1 = require('../../utils/log');

const url_1 = require('./url');

const fetch_1 = require('../../utils/fetch');

let instance = new APIGateway_1.default();

class HttpClient {
  constructor(API_RETRY_TIME, API_APIGATEWAY_OPEN, timeout, API_ECONNREFUSED_RETRY_TIME) {
    this.API_RETRY_TIME = API_RETRY_TIME;
    this.API_APIGATEWAY_OPEN = API_APIGATEWAY_OPEN;
    this.timeout = timeout;
    this.API_ECONNREFUSED_RETRY_TIME = API_ECONNREFUSED_RETRY_TIME;

    if (process.env.NODE_ENV === 'development') {
      instance = new APIGateway_1.default();
    }
  }

  async startFetch(
    options,
    retryTime = this.API_RETRY_TIME,
    refusedRetryTiem = this.API_ECONNREFUSED_RETRY_TIME
  ) {
    const { url } = options;

    try {
      const uri = await this.getUri(url);

      if (!/^http(s)?:/.test(uri)) {
        throw {
          status: 500,
          message: `请求地址[${url}]匹配的domain必须设置http协议，当前node请求的完整url为${uri}`,
          fetch: true
        };
      }

      try {
        return await fetch_1
          .default({ ...options, url: uri, timeout: this.timeout })
          .then((response) => {
            if (this.API_RETRY_TIME && retryTime > 0) {
              // 设置了重试次数，且当前重试次数 > 0
              if (response.status < 200 || response.status > 350) {
                throw {
                  type: 1,
                  message: `${response.url}: ${response.statusText}`
                };
              }
            }

            return response;
          })
          .catch((err) => {
            throw err;
          });
      } catch (e) {
        if (e.type === 1) {
          if (retryTime > 0) {
            log_1.default.error(e.message, 'server-fetch-retry');
            return this.startFetch(options, --retryTime);
          }
        } else if (e.code === 'ECONNREFUSED') {
          if (refusedRetryTiem > 0) {
            log_1.default.error(e.message, 'server-fetch-eeconnefused');
            return this.startFetch(options, retryTime, --refusedRetryTiem);
          }
        } else {
          throw e;
        }
      }
    } catch (err) {
      log_1.default.error(err, 'apiGetWay-getUri-error');
      return Promise.reject(err);
    }
  }

  async getApiGatewayUri(url) {
    // 如 是http开头，直接返回url
    if (/^http(s)?:/.test(url)) {
      return url;
    }

    const apiGatewayServerIP = await instance.getApiServer();
    return `http://${apiGatewayServerIP}${url}`;
  }

  getUri(url) {
    if (Number(process.env.Browser) === 1) {
      return Promise.resolve(url);
    }

    if (this.API_APIGATEWAY_OPEN) {
      return this.getApiGatewayUri(url);
    }

    return Promise.resolve(url_1.getFullUrl(url));
  }
}

exports.default = (options) => {
  const { fetch: fetchConfig } = server_1.getAwardConfig();
  const { timeout = 5000 } = fetchConfig;
  const _apiGatewayConfig = { ...apiGatewayConfig, ...fetchConfig.apiGateway };
  const { API_RETRY_TIME, API_APIGATEWAY_OPEN, API_ECONNREFUSED_RETRY_TIME } = _apiGatewayConfig;
  const httpClient = new HttpClient(
    API_RETRY_TIME,
    API_APIGATEWAY_OPEN,
    timeout,
    API_ECONNREFUSED_RETRY_TIME
  ); // __headers 为后续服务端带cookie请求数据留
  // options.headers = Object.assign(...options.__headers, options.headers);
  // apiGetWay

  if (API_APIGATEWAY_OPEN) {
    Object.assign(options.headers || {}, {
      Host: url_1.getDomain(options.url)
    });
  }

  return httpClient.startFetch({ ...options });
};
