'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const fetch = require('isomorphic-fetch');

const defaultsDeep = require('lodash/defaultsDeep');

const isString = require('lodash/isString');

const some = require('lodash/some');

const transformRequest_1 = require('./transformRequest');

const xhr_1 = require('./xhr');

const interceptors_1 = require('./interceptors');

const log_1 = require('./log');

function parseResponseJSON(response) {
  try {
    return xhr_1.checkStatus(response).json();
  } catch (error) {
    return response;
  }
}

function parseResponseText(response) {
  try {
    return xhr_1.checkStatus(response).text();
  } catch (error) {
    return response;
  }
}

function parseObject(data) {
  if (typeof data === 'string') {
    try {
      return JSON.parse(data);
    } catch (error) {}
  }

  return data;
}

function parseXhrJSON(data) {
  if (typeof data === 'string') {
    try {
      return JSON.parse(data);
    } catch (error) {}
  }

  return data;
}

const defaultOptions = {
  method: 'get',
  dataType: 'json',
  // 允许跨域
  credentials: 'include',
  mode: 'cors',
  transformRequest: transformRequest_1.default,
  transformResponse: parseResponseJSON,
  headers: {
    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8'
  }
};
/**
 * 是否需要xhr（使用了不支持fetch的功能）
 */

function needXHR(opt) {
  return (
    process.env.RUN_ENV === 'web' &&
    some(['xhr', 'onDownloadProgress', 'onUploadProgress', 'timeout'], (v) => opt[v])
  );
}

function mergeOptions(_options, defaultOpt) {
  var _a;

  const existContentType =
    (_a = _options === null || _options === void 0 ? void 0 : _options.headers) === null ||
    _a === void 0
      ? void 0
      : _a['Content-Type'];
  const options = defaultsDeep(_options, defaultOpt);
  const { method, body, params, data, transformRequest: transform, dataType } = options;

  if (method.toLowerCase() === 'get') {
    // get请求支持数据传递，优先级 params > data > body
    const temp = transform(params || data || body);
    options.params = temp ? `?${temp}` : '';
    delete options.body;
  } else {
    // 非get请求，params get数据
    // body 请求体，优先级 data > body
    const temp = transform(params);
    options.params = temp ? `?${temp}` : '';
    options.body = transform(data || body, dataType);

    if (!existContentType) {
      // 如果开发者没有指定contentType，那么走逻辑默认的
      if (!isString(options.body)) {
        // 文件等默认识别content-type
        delete options.headers['Content-Type'];
      } else if (dataType === 'json') {
        options.headers['Content-Type'] = 'application/json';
      }
    }
  }

  if (needXHR(options) && dataType === 'json') {
    options.transformResponse = parseXhrJSON;
  }

  if (dataType === 'string' || dataType === 'text') {
    options.transformResponse = parseResponseText;
  }

  if (dataType === 'object') {
    options.transformResponse = parseObject;
  }

  return options;
}
/**
 * web client fetch
 * @param  {[object]} options fetch参数
 */

exports.default = (options) => {
  const opts = mergeOptions(options, defaultOptions);
  const { url, params, transformResponse } = opts;

  if (needXHR(opts)) {
    return xhr_1.default(opts);
  }

  return fetch(`${encodeURI(url)}${params}`, opts).then(async (response) => {
    let transformData = await transformResponse(response);
    const rl = interceptors_1.interceptors.response.length;

    if (rl) {
      let i = 0;

      while (i < rl) {
        const result = await interceptors_1.interceptors.response[i].func(
          transformData,
          response,
          log_1.default
        );

        if (result) {
          transformData = result;
        }

        i++;
      }
    }

    return transformData;
  });
};
